<?php

namespace Drupal\ckmoon\Form;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Template\Attribute;
use Drupal\editor\Ajax\EditorDialogSave;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\filter\Entity\FilterFormat;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a CKMoon Dialog form.
 */
class CkMoonDialog extends FormBase {

  /**
   * Configuration Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   Defines the configuration object factory.
   */
  public function __construct(ConfigFactory $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ck_moon_dialog';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, FilterFormat $filter_format = NULL) {

    $theme = $this->configFactory->get('system.theme')->get('default');
    $path = drupal_get_path('theme', $theme);
    $editor = editor_load($filter_format->id());
    $settings = $editor->getSettings();

    if (empty($settings['plugins']['ckmoon']['ckmoon']['json'])) {
      $form['no_json'] = [
        '#markup' => $this->t('No json config found in editor settings.'),
      ];
      return $form;
    }

    $jsonPath = $settings['plugins']['ckmoon']['ckmoon']['json'];
    $json = json_decode(file_get_contents($path . '/' . $jsonPath));

    $form['#attached']['library'][] = 'ckmoon/ckmoon';
    // Attach custom theme library to render the icons in admin section.
    $form['#attached']['library'][] = $theme . '/ckmoon';

    $form['icons'] = [
      '#type' => 'fieldset',
      '#attributes' => [
        'class' => [
          'ckmoon__icon-fieldset',
        ],
      ],
    ];

    foreach ($json->icons as $icon) {
      $name = $icon->properties->name;
      $width = isset($icon->icon->width) ? $icon->icon->width : 1024;
      $height = isset($icon->icon->height) ? $icon->icon->height : 1024;

      $prevSize = isset($icon->properties->prevSize) ? $icon->properties->prevSize : FALSE;

      $svg_attributes = new Attribute([
        'class' => ['ckmoon__icon'],
        'width' => $prevSize ?: $width,
        'height' => $prevSize ?: $height,
        'viewbox' => "0 0 {$width} {$height}",
      ]);

      $paths = [];
      foreach ($icon->icon->paths as $path) {
        $paths[] = [
          'attributes' => new Attribute(['d' => $path]),
        ];
      }

      $form['icons'][$name] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'ckmoon__icon-container',
          ],
        ],
        'preview' => [
          '#theme' => 'ckmoon_svg',
          '#svg_attributes' => $svg_attributes,
          '#paths' => $paths,
        ],
      ];

      $form['icons'][$name]['button'] = [
        '#type' => 'button',
        '#value' => $name,
        '#attributes' => [
          'class' => [
            $name,
            'ckmoon__icon-button',
          ],
        ],
        '#submit' => [],
        '#ajax' => [
          'callback' => '::submitForm',
          'event' => 'click',
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new EditorDialogSave($form_state->getTriggeringElement()['#value']));
    $response->addCommand(new CloseModalDialogCommand());
    return $response;
  }

}
